@@ -1,4 +1,5 @@
 from .plotter import Plotter
-from .reader import Reader
+from .datafile import DataFile
 from . import writer
 from .database import Database
+from .misc import *
